@@ -12,7 +12,7 @@ module Distribution.Solver.Modular.Version
     ) where
 
 import qualified Distribution.Version as CV -- from Cabal
-import Distribution.Text -- from Cabal
+import Distribution.Deprecated.Text -- from Cabal
 
 -- | Preliminary type for versions.
 type Ver = CV.Version
